Package: AUCell
Type: Package
Title: AUCell: Analysis of 'gene set' activity in single-cell RNA-seq
        data (e.g. identify cells with specific gene signatures)
Version: 1.34.0
Date: 2024-03-09
Author: Sara Aibar, Stein Aerts. Laboratory of Computational Biology.
    VIB-KU Leuven Center for Brain & Disease Research. Leuven, Belgium.
Maintainer: Gert Hulselmans <Gert.Hulselmans@kuleuven.be>
Description: AUCell allows to identify cells with active gene sets
    (e.g. signatures, gene modules...) in single-cell RNA-seq data.
    AUCell uses the "Area Under the Curve" (AUC) to calculate whether a critical
    subset of the input gene set is enriched within the expressed genes for each cell.
    The distribution of AUC scores across all the cells allows exploring the
    relative expression of the signature. Since the scoring method is ranking-based,
    AUCell is independent of the gene expression units and the normalization procedure.
    In addition, since the cells are evaluated individually, it can easily be
    applied to bigger datasets, subsetting the expression matrix if needed.
URL: http://scenic.aertslab.org
Imports: DelayedArray, DelayedMatrixStats, data.table, graphics,
        grDevices, GSEABase, Matrix, methods, mixtools, R.utils, stats,
        SummarizedExperiment, BiocGenerics, utils
Enhances: doMC, doRNG, doParallel, foreach
Suggests: Biobase, BiocStyle, doSNOW, dynamicTreeCut, DT, GEOquery,
        knitr, NMF, plyr, R2HTML, rmarkdown, reshape2, plotly, Rtsne,
        testthat, zoo
License: GPL-3
biocViews: SingleCell, GeneSetEnrichment, Transcriptomics,
        Transcription, GeneExpression, WorkflowStep, Normalization
VignetteBuilder: knitr
RoxygenNote: 7.2.0
git_url: https://git.bioconductor.org/packages/AUCell
git_branch: RELEASE_3_23
git_last_commit: 825bca6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 20:26:37 UTC; biocbuild
