# AUCell
AUCell is an R-package to analyze the state of gene-sets in single-cell RNA-seq data (i.e. identify cells with active gene signatures).

*Availability*: 

- The newest stable version of AUCell is available in *[Bioconductor](https://bioconductor.org/packages/AUCell)*. 
The package also contains a **tutorial** [(vignette)](https://bioconductor.org/packages/release/bioc/vignettes/AUCell/inst/doc/AUCell.html) with information on how to run AUCell and how to interprete its results.

- This Github repository is meant mostly for development. "Use at your own risk" :-). Developers are welcome to contribute. 
